; ------------------------------------------------------
; Metalwork (Shader Model 3.0 version)
; Written by Franck "hitchhikr" Charlet / Neural
; ------------------------------------------------------
; Thanks to rbraz for the trianglelist trick.
; ------------------------------------------------------
; buildblock RELEASE
; CAPT "[SOURCEDIR]nasmw.exe" -f bin "%2" -o "%1.bin" -s -O9
; CAPT "[SOURCEDIR]1kpack.exe" "depackers\dx_depacker.bin" "%1.bin"
; buildblockend
;
; buildblock DEBUG
; CAPT "[SOURCEDIR]nasmw.exe" -D DEBUG -f bin "%2" -o "%1.bin" -s -O9
; CAPT "[SOURCEDIR]1kpack.exe" "depackers\dx_depacker_debug.bin" "%1.bin"
; buildblockend

; ------------------------------------------------------
; Header
                                bits    32

; ------------------------------------------------------
; Includes
                                %include "d3d9_nasm.inc"

; ------------------------------------------------------
; Pre-processor constants
;%define DEBUG

; ------------------------------------------------------
; User constants
NOISE_WIDTH                     equ     14

SCREENX                         equ     1024
SCREENY                         equ     768

; ------------------------------------------------------
; Constants
WS_EX_TOPMOST                   equ     0x8
VK_ESCAPE                       equ     0x1b
WS_CAPTION                      equ     0xc00000
WS_POPUP                        equ     0x80000000
WS_BORDER                       equ     0x800000
WS_SYSMENU                      equ     0x80000
WS_VISIBLE                      equ     0x10000000
WS_THICKFRAME                   equ     0x40000
WS_POPUPWINDOW                  equ     WS_BORDER | WS_SYSMENU | WS_CAPTION | WS_VISIBLE | WS_THICKFRAME
SW_SHOWNORMAL                   equ     1

PM_REMOVE                       equ     0x1

SCREENDEPTH                     equ     D3DFMT_A8R8G8B8     ; 32 bits

D3DFVF_XYZRHW_SIZE              equ     (4 * 4)

BASE                            equ     0x420000

; Those are located inside the depacker
POS_fnc_exit                    equ     0x410090
POS_fnc_D3DXCompileShader       equ     0x410094
POS_fnc_LoadLibrary             equ     0x4100b4
POS_fnc_GetProcAddress          equ     0x4100b8
POS_fnc_QueryPerformanceCounter equ     0x4100bc

; !!! Only valid for d3dx9_30.dll !!!
D3DX_D3DXCompileShader          equ     0xda6de
D3DX_D3DXCreateTextureFromFileInMemoryEx equ    (D3DX_D3DXCompileShader - 0xc1341)
D3DX_D3DXCreateTexture          equ     (D3DX_D3DXCompileShader - 0xbe4a2)

                                %ifdef DEBUG
NBR_TOP_FUNCTIONS               equ     5
                                %else
NBR_TOP_FUNCTIONS               equ     3
                                %endif

; ------------------------------------------------------
; Structures

; ------------------------------------------------------
; Vars datas
;
; Note: OldTime & HTimerFreq MUST be 64 bits aligned
;       otherwise QueryPerformanceCounter & QueryPerformanceFrequency
;       won't work properly, so check the alignment of Vars: at the
;       end of this file.
;
Device                          equ     0
OldTime                         equ     Device + 4
HTimerFreq                      equ     OldTime + 8
PixelShader                     equ     HTimerFreq + 8
PixelShader2                    equ     PixelShader + 4
CurShader                       equ     PixelShader2 + 4
Texture_Noise                   equ     CurShader + 4
Shader_FlipFlop                 equ     Texture_Noise  + 4
Sens_Fade                       equ     Shader_FlipFlop + 4
Scroll_Pos                      equ     Sens_Fade + 4
Fade                            equ     Scroll_Pos + 4

; ---------------------------------------------------------
; Program entry point
; edi contains D3DXCompileShader address
                                org     BASE

EntryPoint:                     sub     edi, D3DX_D3DXCreateTextureFromFileInMemoryEx
                                push    edi

                                ; Import the functions we need
                                mov     edi, table_dlls
load_dlls:                      push    edi
                                call    dword [POS_fnc_LoadLibrary]
                                test    eax, eax
                                jz      done_dlls
                                push    -1                              ; Fix a problem with windows 7
                                pop     ecx
                                repne   scasb                           ; Go to end of string
load_functions:                 xchg    ebx, eax                        ; DLL handle
                                push    edi
                                push    ebx
                                call    dword [POS_fnc_GetProcAddress]
                                test    eax, eax
                                jz      load_dlls
                                stosd
                                xchg    ebx, eax
                                repne   scasb                           ; Go to end of string
                                jmp     load_functions
done_dlls:
                                ; ----------------------------------------------------

                                mov     esi, API_CreateWindowEx
                                mov     edi, POS_fnc_QueryPerformanceCounter
                                mov     ebp, Vars

                                push    ebp                             ; CreateDevice
                                push    Present_Buffer
                                push    D3DCREATE_SOFTWARE_VERTEXPROCESSING

                                push    D3D_SDK_VERSION                 ; Direct3DCreate9

                                lea     eax, [ebp + PixelShader2]
                                push    0                               ; D3DXCompileShader
                                push    0
                                push    eax
                                push    0
                                push    PShaderProfileName
                                push    ProcedureName
                                push    0
                                push    0
                                push    (fPShader2 - PShader2)
                                push    PShader2

                                lea     eax, [ebp + PixelShader]
                                push    0                               ; D3DXCompileShader
                                push    0
                                push    eax
                                push    0
                                push    PShaderProfileName
                                push    ProcedureName
                                push    0
                                push    0
                                push    (fPShader - PShader)
                                push    PShader

                                %ifndef DEBUG
                                push    0                               ; SetCursor
                                %endif

                                push    0                               ; CreateWindowEx
                                push    0
                                push    0
                                push    0
                                %ifdef DEBUG
                                push    SCREENY
                                push    SCREENX
                                %else
                                push    0
                                push    0
                                %endif
                                push    0
                                push    0
                                %ifdef DEBUG
                                push    WS_POPUPWINDOW
                                %else
                                push    0
                                %endif
                                push    0
                                push    ClassName
                                %ifdef DEBUG
                                push    0
                                %else
                                push    WS_EX_TOPMOST
                                %endif
                                call    dword [esi]                     ; + (API_CreateWindowEx - API_CreateWindowEx)]
                                mov     ebx, eax
                                %ifdef DEBUG
                                push    SW_SHOWNORMAL
                                push    eax
                                call    dword [esi + (API_ShowWindow - API_CreateWindowEx)]
                                %else
                                call    dword [esi + (API_SetCursor - API_CreateWindowEx)]
                                %endif

                                call    dword [edi + (POS_fnc_D3DXCompileShader - POS_fnc_QueryPerformanceCounter)]
                                call    dword [edi + (POS_fnc_D3DXCompileShader - POS_fnc_QueryPerformanceCounter)]

                                ; ------------------------------------------------------
                                call    dword [esi + (API_Direct3DCreate9 - API_CreateWindowEx)]

                                push    ebx
                                push    D3DDEVTYPE_HAL
                                push    D3DADAPTER_DEFAULT              ; (0)
                                push    eax
                                mov     ebx, [eax]
                                call    [ebx + IDirect3D9.CreateDevice]

                                lea     eax, [ebp + PixelShader]
                                push    eax
                                mov     eax, [eax]
                                push    eax
                                mov     ebx, [eax]
                                call    [ebx + ID3DXConstantTable.GetBufferPointer]
                                push    eax
                                mov     eax, [ebp]
                                push    eax
                                mov     ebx, [eax]
                                call    [ebx + IDirect3DDevice9.CreatePixelShader]

                                lea     eax, [ebp + PixelShader2]
                                push    eax
                                mov     eax, [eax]
                                push    eax
                                mov     ebx, [eax]
                                call    [ebx + ID3DXConstantTable.GetBufferPointer]
                                push    eax
                                mov     eax, [ebp]
                                push    eax
                                mov     ebx, [eax]
                                call    [ebx + IDirect3DDevice9.CreatePixelShader]

                                pop     ebx                             ; ebx = D3DXCreateTextureFromFileInMemoryEx

                                lea     eax, [ebp + Texture_Noise]
                                push    eax
                                push    0
                                push    0
                                push    0
                                push    D3DTEXF_ANISOTROPIC
                                push    D3DTEXF_ANISOTROPIC
                                push    D3DPOOL_DEFAULT
                                push    0
                                push    0
                                push    0
                                push    0
                                push    0
                                push    512
                                push    Noise
                                
                                mov     eax, [ebp]
                                push    eax
                                call    ebx

                                mov     eax, [ebp + PixelShader]
                                mov     [ebp + CurShader], eax

                                lea     eax, [ebp + OldTime]
                                push    eax
                                call    dword [edi + (POS_fnc_QueryPerformanceCounter - POS_fnc_QueryPerformanceCounter)]

; ------------------------------------------------------
; Program loop
MainLoop:                       
                                %ifdef DEBUG
                                push    PM_REMOVE
                                push    0
                                push    0
                                push    0
                                push    Msg
                                call    dword [esi + (API_PeekMessage - API_CreateWindowEx)]
                                test    eax, eax
                                jz      No_Msg
                                push    Msg
                                call    dword [esi + (API_TranslateMessage - API_CreateWindowEx)]
No_Msg:
                                %endif

                                ; ------------------------------------------------------
                                ; Obtain the frames timer
                                fild    qword [ebp + OldTime]
                                lea     eax, [ebp + OldTime]
                                push    eax
                                call    dword [edi + (POS_fnc_QueryPerformanceCounter - POS_fnc_QueryPerformanceCounter)]
                                fild    qword [ebp + OldTime]
                                fsubp   st1, st0
                                lea     eax, [ebp + HTimerFreq]
                                push    eax
                                call    dword [esi + (API_QueryPerformanceFrequency - API_CreateWindowEx)]
                                fild    qword [ebp + HTimerFreq]
                                fdivp   st1, st0

                                ; ------------------------------------------------------
                                ; Animate
                                fchs
                                fmul    dword [Scroll_Speed]
                                fld     st0
                                fadd    dword [ebp + Scroll_Pos]
                                fcom    dword [Time_Effects]
                                fnstsw  ax
                                sahf
                                setnb   byte [ebp + Sens_Fade]
                                fstp    dword [ebp + Scroll_Pos]
                                jb      Fade_In                     ; (The flag aren't changed before this) 
                                fchs
                                fadd    dword [ebp + Fade]
                                ftst
                                fnstsw  ax
                                sahf
                                ja      Fade_Out
                                ; Change shader when screen is black
                                mov     eax, [ebp + PixelShader]
                                xor     dword [ebp + Shader_FlipFlop], 1
                                jz      Set_Shader_2
                                mov     eax, [ebp + PixelShader2]
Set_Shader_2:                   mov     [ebp + CurShader], eax
                                fsub    st0, st0
                                fst     qword [ebp + Sens_Fade]     ; Qword to clear Scroll_Pos at the same time
                                fst     qword [ebp + Fade]          ; For redundancy purpose only
Fade_In:                        fadd    dword [ebp + Fade]
Fade_Out:                       fstp    dword [ebp + Fade]

                                ; ------------------------------------------------------
                                ; Display a rectangle
                                lea     ecx, [ebp + Scroll_Pos]

                                push    VK_ESCAPE                       ; GetAsyncKeyState

                                push    0                               ; Present
                                push    0
                                push    0
                                push    0
                                mov     eax, [ebp]
                                push    eax
                                mov     ebx, [eax]

                                push    eax                             ; EndScene

                                push    D3DFVF_XYZRHW_SIZE              ; DrawPrimitiveUP
                                push    BigTriangle
                                push    1
                                push    D3DPT_TRIANGLELIST
                                push    eax

                                push    dword [ebp + Texture_Noise]     ; SetTexture
                                push    0
                                push    eax

                                push    D3DFVF_XYZRHW                   ; SetFVF
                                push    eax

                                push    dword [ebp + CurShader]         ; SetPixelShader
                                push    eax

                                push    1                               ; (Nbr) SetPixelShaderConstantF
                                push    ecx
                                push    0                               ; First constant register number
                                push    eax

                                push    eax
                                call    [ebx + IDirect3DDevice9.BeginScene]
                                call    [ebx + IDirect3DDevice9.SetPixelShaderConstantF]
                                call    [ebx + IDirect3DDevice9.SetPixelShader]
                                call    [ebx + IDirect3DDevice9.SetFVF]
                                call    [ebx + IDirect3DDevice9.SetTexture]
                                call    [ebx + IDirect3DDevice9.DrawPrimitiveUP]
                                call    [ebx + IDirect3DDevice9.EndScene]
                                call    [ebx + IDirect3DDevice9.Present]
                                call    dword [esi + (API_GetAsyncKeyState - API_CreateWindowEx)]
                                sahf
                                jns     MainLoop
                                call    dword [edi + (POS_fnc_exit - POS_fnc_QueryPerformanceCounter)]

; ------------------------------------------------------
; Datas
                                %ifdef DEBUG
Msg:                            times   100 db 0
                                %endif

ClassName:                      db      "edi"
ProcedureName:                  db      "t", 0
PShaderProfileName:             db      "ps_3_0", 0

Noise:                          db      0
                                db      0
                                db      3
                                db      0, 0, 0, 0, 0
                                dw      0
                                dw      0
                                dw      NOISE_WIDTH
                                dw      NOISE_WIDTH
                                db      8
                                db      0

; Raycasted tunnel
PShader2:                       db      "sampler z;"
                                db      "float4 a;"
                                db      "float4 t(float2 x:vpos):color"
                                db      "{"
                                db      "x.x/=512;"
                                db      "x.y/=384;"
                                db      "x-=1;"
                                db      "float4 y=2/dot(x,x)*float4(x.x,x.y,.2,2);"
                                db      "y.y=atan2(y.y,y);"
                                db      "y.x=y.z+a.x*.2;"
                                db      "y=tex2D(z,y)*y.z;"
                                db      "y.xy*=(2/((sin(a+x)*1.5+3)*dot(x,x)))*.3;"
                                db      "return y*clamp(a.y,0,1);"
                                db      "}"
fPShader2:

; Some metallic shit
PShader:                        db      "sampler z;"
                                db      "float4 a;"
                                db      "float4 t(float2 x:vpos):color"
                                db      "{"
                                db      "x.x/=512;"
                                db      "x.y/=384;"
                                db      "x-=1;"
                                db      "float4 y=atan2(x.x,x.y);"
                                db      "y=sin((length(x)+2*y.x)*22-2*a.x)-2*sin(y.x);"
                                db      "y=tex2D(z,y*.2+x)*y.z;"
                                db      "y.xy+=x.x*.05;"
                                db      "y.z+=.05;"
                                db      "return y*clamp(a.y,0,1);"
                                db      "}"
fPShader:

Scroll_Speed:                   dd      1.5
Time_Effects:                   dd      12.0

                                ; -------------------------------
table_dlls:                     ; 3 functions
                                db      "user32", 0
API_CreateWindowEx:             db      "CreateWindowExA", 0
                                %ifdef DEBUG
API_ShowWindow:                 db      "ShowWindow", 0
                                %else
API_SetCursor:                  db      "SetCursor", 0
                                %endif
API_GetAsyncKeyState:           db      "GetAsyncKeyState", 0
                                %ifdef DEBUG
API_PeekMessage:                db      "PeekMessageA", 0
API_TranslateMessage:           db      "TranslateMessage", 0
                                %endif
                                ; 2 functions
                                db      "d3d9", 0
API_Direct3DCreate9:            db      "Direct3DCreate9", 0
                                ; 1 function
                                db      "kernel32", 0
API_QueryPerformanceFrequency:  db      "QueryPerformanceFrequency", 0

Present_Buffer:                 dd      SCREENX                     ; dpBackBufferWidth
                                dd      SCREENY                     ; dpBackBufferHeight
                                dd      SCREENDEPTH                 ; dpBackBufferFormat
                                dd      0                           ; dpBackBufferCount
                                dd      0                           ; dpMultiSampleType
                                dd      0                           ; dpMultiSampleQuality
BigTriangle:                    dd      D3DSWAPEFFECT_DISCARD       ; dpSwapEffect
                                dd      0                           ; dphDeviceWindow
                                ; dpWindowed / Windowed = 1 / Fullscreen = 0
                                %ifdef DEBUG
                                dd      1
                                %else
                                dd      0
                                %endif
                                dd      0                           ; dpEnableAutoDepthStencil
                                dd      2048.0                      ; dpAutoDepthStencilFormat (D3DFMT_D16)
                                dd      0                           ; dpFlags
                                dd      0                           ; dpFullScreen_RefreshRateInHz
                                dd      0                           ; dpFullScreen_PresentationInterval
                                dd      0
                                dd      2048.0

                                ; Note: the packer strips any 0 located
                                ; at the end of the file (including the flt above)
                                ; this can eventually be used as a bss section

                                align   8
Vars:
; ------------------------------------------------------
